/*

    File: CDPlayer.cpp

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "../m_system3d/main.h"
#include "../m_foundation/main.h"
#include "../m_foundation/const.h"
#include "../m_cdplayer/cdplayer.h"
#include "cdplayer.h"

static M_CDPlayer *main_cdPlayer = NULL;

static M_Camera cdPlayer_camera;

static bool main_cdPlayer_visible;

void CFunc_ToggleCDPlayer(int,char**)
{
	main_cdPlayer_visible = !main_cdPlayer_visible;
}

bool Init_CDPlayer(void)
{
//init the cdplayer (cdaudio GUI)
	main_cdPlayer = new M_CDPlayer();
	if (main_cdPlayer && !main_cdPlayer->Init())
	{
		delete main_cdPlayer;
		main_cdPlayer = NULL;
		main_console->Print("Error in creating CD Player object\n");
	}
	if (!main_cdPlayer) return false;

	//cdplayer stats
	VectorSet(&main_cdPlayer->position, 9000,40,40);
	main_cdPlayer->useTranslation = false;
	main_cdPlayer->rotationConst = 50;

	main_cdPlayer_visible = false;

	//cvars
	new CVarw(0, "CDPlayer", CFunc_ToggleCDPlayer);

	cdPlayer_camera.ortho = true;
	cdPlayer_camera.halfWidth = 50;
	cdPlayer_camera.halfHeight = 50;
	cdPlayer_camera.adjustRatio = CAMERA_ADJUST_WIDTH;
	cdPlayer_camera.znear = 1;
	cdPlayer_camera.zfar = 10000;
	VectorCopy(&vectorZero, &cdPlayer_camera.position);
	QuatCopy(&quatIdentity, &cdPlayer_camera.angle);

	//only need to calc frustum once as long as it is a stationary camera
	// hint hint: implement the camera system to ONLY UPDATE FRUSTUM WHEN POSITION/ANGLE HAS CHANGED
	cdPlayer_camera.CalculateFrustum();

	return true;
}

void PreUpdate_CDPlayer(void)
{
	if (main_cdPlayer_visible)
	{
		cdPlayer_camera.CalcMouseLine();

		{
			M_Camera *pushCamera = main_camera;
			main_camera = &cdPlayer_camera;

			//calculate the mouseLine before executing main_cdplayer->PreUpdate
			main_cdPlayer->PreUpdate();

			main_camera = pushCamera;
		}
	}
}

void PostUpdate_CDPlayer(void)
{
	if (main_cdPlayer_visible)
	{
		main_cdPlayer->PostUpdate();

		//push and create new matrix system
		{
			glMatrixMode(GL_PROJECTION);
			glPushMatrix();
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();

			cdPlayer_camera.TransformScene(CAMERA_RESET_PROJECTION | CAMERA_RESET_ROTATION);
		}

		glColor3f(1,1,1);

		glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_FOG);
		glEnable(GL_LIGHTING);

		main_cdPlayer->Display();

		glPopAttrib();

		//pop old matrix system
		{
			glPopMatrix();
			glMatrixMode(GL_PROJECTION);
			glPopMatrix();
			glMatrixMode(GL_MODELVIEW);
		}
	}
}

void Shutdown_CDPlayer(void)
{
	if (main_cdPlayer)	{	delete main_cdPlayer;	main_cdPlayer = NULL;	}
}

