/*

    File: CModel.h

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef CMODEL_H
#define CMODEL_H

#include "../m_foundation/chain.h"
#include "../m_math3d/geometry.h"

typedef struct edge_s {
	int t[2];	//two triangle IDs
	int v[2];	//two vtx IDs
} edge_t;

class CModel : public Chain {
public:
	CModel()	{Reset();}
	~CModel()	{Unload();}

	bool		Load(char *filename);
	void		Unload(void);
	void		Reset(void);

	friend CModel *GetCModel(LinkedList *p, char *title);

	char		title[64];

	int			vtxCount;
	vector_t	*vtx;		//vtxCount

	int			planeCount;
	plane_t		*plane;		//triCount

	int			edgeCount;
	edge_t		*edge;		//edgeCount

	int			triCount;
	int			*tri;		//3*triCount
	vector_t	*triNormal;	//triCount

	box_t		bbox;
	scalar_t	max_radius;		//maximum magnitude vertex distance - used for occulsion / collision testing
	scalar_t	min_radius;		//minimum magnitude vertex distance - same deal
	scalar_t	volume;		//calculated volume of the model
	vector_t	axisMoment;

	void CalcInertia(const vector_t *scale, vector_t *vres);
};

#endif

//second system involves triangle array / triangle normal array / no edge array