/*

    File: Collision.h

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef COLLISION_H
#define COLLISION_H

typedef struct collision_info_s collision_info_t;
typedef struct collision_result_s collision_result_t;

#include "entity.h"

	//pre:
struct collision_info_s {
	vector_t	point;		//point of collision between 2 objs
	vector_t	normal;		//normal vector from obj[1] toward obj[0]
	entity_t	*obj[2];	//the two objects which are colliding
	vector_t	pos[2];		//the location considered for obj[0] and obj[1] (may be dif than obj[n]->position due to modulospace)
};
	//post:
struct collision_result_s {
	scalar_t impulse_mag;	//magnitude of resultant force
	vector_t impulse;		//resultant force enacted upon both objects
};

#define C_MODEL_OBJ0		0
#define C_MODEL_OBJ1		1

//used to load collision models C_MODEL_OBJ0 and C_MODEL_OBJ1
//once each is loaded, collision between cmodels can be tested
void LoadCollisionModel(
	const CModel *c_model,
	const vector_t *position,
	const quat_t *angle,
	const vector_t *scale,
	int cid);

/*
tests for a collison between the two loaded collision models

  if collision does occur
	fills contents of res_point with collision point
	fills contents of res_normal with collision normal
	returns 1

  if collision does not occur
	fills contents of witness
	returns 0
*/
int ObjectCollision(vector_t *res_point, vector_t *res_normal, plane_t *witness);

/*
tests whether the witness plane separates the two loaded collision models
returns 1 if they are separated
returns 0 if not
*/
int WitnessSeparates(const plane_t *witness);


//applies any ol' force you specify
void ApplyForce(entity_t *e, const vector_t *point, const vector_t *force);

//applies the resulting collision forces based upon collision information generated at a Touch function
void ApplyImpulseForces(const collision_info_t *info, collision_result_t *result);

#endif