/*

    File: Main.cpp

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include "../m_foundation/main.h"

#include "entity.h"
#include "lighting.h"
#include "cdplayer.h"
#include "main.h"
#include "menu.h"
#include "particle.h"

M_World *world = NULL;

int game_Active = GAMEACTIVE_YES;

void CFunc_Pause(int, char **)
{
	switch (game_Active)
	{
	case GAMEACTIVE_YES:
		game_Active = GAMEACTIVE_PAUSED;
		break;
	case GAMEACTIVE_PAUSED:
		game_Active = GAMEACTIVE_YES;
		break;
	}
}

int game_State = 0;

bool Init_Application3D(void)
{
#if 0
found_GL_ARB_texture_cube_map = false;
found_GL_ARB_multitexture = false;
found_GL_EXT_draw_range_elements = false;
found_GL_EXT_compiled_vertex_array = false;
found_GL_EXT_point_parameters = false;
found_GL_EXT_separate_specular_color = false;
found_GL_NV_texgen_reflection = false;
found_GL_NV_light_max_exponent = false;
found_GL_NV_texgen_emboss = false;
found_GL_NV_vertex_array_range = false;
#endif

	//the initial OpenGL settings...
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	main_console->SetIdleVisible(false);
	CVar_Exec("ConsoleFont WorldData/font.bmp");
	main_camera->zfar = 2000;

	//a better angle of it all...
	VectorSet(&main_camera->position, -50, 0, 30);

	if (!Init_CDPlayer()) return false;

//init the world
	world = new M_World();
	if (world && !world->Init())
	{
		delete world;
		world = NULL;
		main_console->Print("Error in creating world\n");	//tell the world something went wrong
	}
	if (!world) return false;

//get the particle system
	if (!Init_ParticleSystem(5000))
	{
		main_console->Print("Error in creating particle system\n");	//tell the world something went wrong
		return false;
	}

//init entity system
	if (!InitEntities()) return false;
//init lighting system (fix me up - i am so freakin weak!)
	Init_Lighting();
//and the menu system
	if (!Init_Menu()) return false;

	new CVarw(0, "Pause", CFunc_Pause);

	game_State = GAMESTATE_OPEN_MENU;

	return true;
}

#include "../m_system3d/main.h"
void Update_Brawl(void)
{
	//display first - order matters, and it is currently a mess
	Update_Lighting();

	world->Display();
	DisplayEntities();
	Display_ParticleSystem();


	if (game_Active == GAMEACTIVE_YES)
	{
		Reset_Lighting();
		UpdateEntities();
		Update_ParticleSystem();
	}

	static int useForcedFPS = 1;
	static double desiredFPS = 40.0;
	static CVarw *dfjlksfd[] = {
		new CVarw(&desiredFPS, "desiredFPS"),
		new CVarw(&useForcedFPS, "useForcedFPS"),
		NULL};

	if (useForcedFPS)
	{
		static double lastTime = 0;
		double thisTime;
		double desiredDT = 1.0 / desiredFPS;
		do {
			thisTime = main_timer->GetTime();
		} while ((thisTime - lastTime) < desiredDT);
		lastTime = thisTime;
	}
}

void Update_Application3D(void)
{
	//the intro to my lil cdplayer wrapper
	PreUpdate_CDPlayer();

	switch (game_State)
	{
	case GAMESTATE_OPEN_MENU:
		Open_Menu();
		game_State = GAMESTATE_MENU;
		break;

	case GAMESTATE_MENU:
		if (main_keyboard->keystate[VK_ESCAPE] == KBH_PRESS) 
			game_State = GAMESTATE_BRAWL;

		//i should have you know that i really really really hate dup code
		//esp when it doesnt need to be

		{
			Update_Lighting();		//lights must be first
			world->Display();
			DisplayEntities();
			Display_ParticleSystem();
		}

		Update_Menu();
		break;

	case GAMESTATE_NEWGAME:

		//tell the entity system to reset the world *here*
		EntityScript_Reset();

		//start us a game.  (this var is messed)
		game_Active = GAMEACTIVE_YES;

		//and get into the loop
		game_State = GAMESTATE_BRAWL;

		break;

	case GAMESTATE_BRAWL:
		if (main_keyboard->keystate[VK_ESCAPE] == KBH_PRESS) 
			game_State = GAMESTATE_OPEN_MENU;

		Update_Brawl();
		break;
	}

	//and the exit to the cdplayer wrapper
	PostUpdate_CDPlayer();
}

void Shutdown_Application3D(void)
{
	Shutdown_Menu();
	Shutdown_Lighting();
	ShutdownEntities();
	Shutdown_ParticleSystem();

	if (world)			{	delete world;			world = NULL;			}

	Shutdown_CDPlayer();
}