/*

    File: World.cpp

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#include <windows.h>
#include <gl/gl.h>
#include <math.h>

#include "../m_math3d/geometry.h"
#include "../m_foundation/main.h"
#include "../m_foundation/const.h"

#include "world.h"
#include "menu.h"
#include "main.h"

static vector_t worldGridColor;
static scalar_t worldGridSpacing = 200;//1000;
static int w_DrawSkyCube = 1;
static int w_DrawWorldGrid = 0;
static int skymap_use_cubemap = 1;

void CFunc_UpdateFade(int, char**)
{
	if (!world) return;
	world->UpdateFade();
}


#include <stdio.h>
void CFunc_CS_Sky(int argc, char **argv)
{
	static const char *skytexdir = "c:\\sierra\\half-life\\cstrike\\gfx\\env\\";

	static char skyfn[7][64];

	if (argc < 2)
	{
		sprintf(skyfn[0], "WorldData\\Background\\sun_512.bmp");
		sprintf(skyfn[1], "WorldData\\Background\\stars_512.bmp");
		sprintf(skyfn[2], "WorldData\\Background\\stars_512.bmp");
		sprintf(skyfn[3], "WorldData\\Background\\stars_512.bmp");
		sprintf(skyfn[4], "WorldData\\Background\\stars_512.bmp");
		sprintf(skyfn[5], "WorldData\\Background\\stars_512.bmp");
	}
	else
	{
		sprintf(skyfn[0], "%s%sft.tga", skytexdir, argv[1]);
		sprintf(skyfn[1], "%s%sbk.tga", skytexdir, argv[1]);
		sprintf(skyfn[2], "%s%slf.tga", skytexdir, argv[1]);
		sprintf(skyfn[3], "%s%srt.tga", skytexdir, argv[1]);
		sprintf(skyfn[4], "%s%sup.tga", skytexdir, argv[1]);
		sprintf(skyfn[5], "%s%sdn.tga", skytexdir, argv[1]);
	}
	sprintf(skyfn[6], "Sky Texture");

	const char *newsky[7] = {skyfn[0], skyfn[1], skyfn[2], skyfn[3], skyfn[4], skyfn[5], skyfn[6]};

	if (!world->skyTex.CreateCubeMap(newsky))
		main_console->Print("unable to create sky cubemap\n");

	for (int i = 0; i < 6; i++)
		//force RGB only so we deny any alpha channels coming thru
		if (!world->backupSkyTex[i].Create(skyfn[i]))
			main_console->Print("unable to create sky textures\n");
}

M_World::M_World() {}

bool M_World::Init(void)
{
	fade_start = main_camera->zfar * 0.8f;
	fade_stop = main_camera->zfar;

	static const char *skyfn[7] = 
	{
		"WorldData\\Background\\sun_512.bmp",
		"WorldData\\Background\\stars_512.bmp",
		"WorldData\\Background\\stars_512.bmp",
		"WorldData\\Background\\stars_512.bmp",
		"WorldData\\Background\\stars_512.bmp",
		"WorldData\\Background\\stars_512.bmp",
		"Sky Texture"
	};


	if (!skyTex.CreateCubeMap(skyfn))
	{
		skymap_use_cubemap = false;
		main_console->Print("unable to create sky cubemap\n");
	}

	for (int i = 0; i < 6; i++)
		if (!backupSkyTex[i].Create(skyfn[i]))
			return false;

	WorldSize = 1500;

	VectorSet(&worldGridColor, .2f,.2f,.2f);

	//due to the worldtree, lets not allow world_size to be dynamic
	//new CVarw(&WorldSize,			"world_size");
	new CVarw(&w_DrawSkyCube,		"DrawSkyCube"),
	new CVarw(&w_DrawWorldGrid,	"DrawWorldGrid"),
	new CVarw(&worldGridColor,		"worldGridColor");
	new CVarw(&worldGridSpacing,	"worldGridSpacing");
	new CVarw(&skymap_use_cubemap,	"skymap_use_cubemap");
	new CVarw(&fade_start,			"world_fade_start",	CFunc_UpdateFade);
	new CVarw(&fade_stop,			"world_fade_stop",	CFunc_UpdateFade);
	new CVarw(0,					"SkyTex",			CFunc_CS_Sky);

	//init gl fog
	UpdateFade();
	
	return true;
}

void M_World::UpdateFade(void)
{
	static float fv0000[4] = {0,0,0,0};

	glEnable(GL_FOG);
	glFogf(GL_FOG_MODE, GL_LINEAR);
	glFogf(GL_FOG_START, fade_start);
	glFogf(GL_FOG_END, fade_stop);
	glFogfv(GL_FOG_COLOR, fv0000);
}

void M_World::Kill(void)
{
	skyTex.Destroy();

	for (int i = 0; i < 6; i++)
		backupSkyTex[i].Destroy();
}

void M_World::DrawSkyCube(void)
{
#define CubeDist 100		//somewhere between main_camera znear and zfar

	static int planes[24] =
	{
		6,4,5,7,
		0,2,3,1,
		2,6,7,3,
		4,0,1,5,
		1,3,7,5,
		4,6,2,0
	};
	
	static float cube_vtxs[8][3] =
	{
		{-CubeDist,	-CubeDist,	-CubeDist	},
		{-CubeDist,	-CubeDist,	CubeDist	},
		{-CubeDist,	CubeDist,	-CubeDist	},
		{-CubeDist,	CubeDist,	CubeDist	},
		{CubeDist,	-CubeDist,	-CubeDist	},
		{CubeDist,	-CubeDist,	CubeDist	},
		{CubeDist,	CubeDist,	-CubeDist	},
		{CubeDist,	CubeDist,	CubeDist	}
	};

	static float backup_cube_vtxs[24][2] =
	{
		{1,0},{1,1},{0,1},{0,0},
		{0,1},{0,0},{1,0},{1,1},
		{0,0},{1,0},{1,1},{0,1},
		{1,1},{0,1},{0,0},{1,0},
		{0,1},{0,0},{1,0},{1,1},
		{0,1},{0,0},{1,0},{1,1}
	};

	//get us the right color
	glColor3f(1,1,1);

	glPushMatrix();
	glLoadIdentity();
	main_camera->TransformScene(CAMERA_RESET_ROTATION);

	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_FOG);
	glDisable(GL_DEPTH_TEST);

	if (found_GL_ARB_texture_cube_map &&	//if the ext is available
		skymap_use_cubemap &&				//if it loaded correctly
		menu_sky_cube_map)					//if the user says so
	{
		glEnable(GL_TEXTURE_CUBE_MAP_ARB);
		glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, skyTex.texture_id);

		glBegin(GL_QUADS);
		for (int i = 0; i < 24; i++)
		{
			glTexCoord3fv(cube_vtxs[planes[i]]);
			glVertex3fv(cube_vtxs[planes[i]]);
		}
		glEnd();
	}
	else
	{
		int i = 0;
		for (int f = 0; f < 6; f++)
		{
			backupSkyTex[f].Set();
			glBegin(GL_QUADS);
			for (int q = 0; q < 4; q++)
			{
				glTexCoord2fv(backup_cube_vtxs[i]);
				glVertex3fv(cube_vtxs[planes[i]]);
				i++;
			}
			glEnd();
		}
	}

	glPopAttrib();
	glPopMatrix();
}

void M_World::DrawWorldGrid(void)
{
	vector_t vec;

	glDisable(GL_TEXTURE_2D);
	glColor3fv(worldGridColor.v);
	glBegin(GL_LINES);
	for (vec.x = -WorldSize; vec.x < WorldSize+1; vec.x += worldGridSpacing)
	{
		for (vec.y = -WorldSize; vec.y < WorldSize+1; vec.y += worldGridSpacing)
		{
			for (vec.z = -WorldSize; vec.z < WorldSize+1; vec.z += worldGridSpacing)
			{
				if (vec.x < WorldSize-1)
				{
					glNormal3f(1,0,0);
					glVertex3fv(vec.v);
					glVertex3f(vec.x + worldGridSpacing, vec.y, vec.z);
				}
				if (vec.y < WorldSize-1)
				{
					glNormal3f(0,1,0);
					glVertex3fv(vec.v);
					glVertex3f(vec.x, vec.y + worldGridSpacing, vec.z);
				}
				if (vec.z < WorldSize-1)
				{
					glNormal3f(0,0,1);
					glVertex3fv(vec.v);
					glVertex3f(vec.x, vec.y, vec.z + worldGridSpacing);
				}
			}
		}
	}
	glEnd();
	glEnable(GL_TEXTURE_2D);
}

void M_World::Display(void)
{
	if (w_DrawSkyCube && menu_sky_enable)
		DrawSkyCube();

	if (w_DrawWorldGrid)
		DrawWorldGrid();
}

//useful functions:

void M_World::Modulo(vector_t *vec)
{
#define MAX_ATTEMPTS	30
	int ctr = 0;

	for (int v = 0; v < 3; v++)
	{
		while (vec->v[v] < -WorldSize)
		{
			vec->v[v] += 2 * WorldSize;
			if (++ctr > MAX_ATTEMPTS)
			{
				vec->v[v] = 0;
				main_console->Print("modulo vector oob\n");
			}
		}
		while (vec->v[v] > WorldSize)
		{
			vec->v[v] -= 2 * WorldSize;
			if (++ctr > MAX_ATTEMPTS)
			{
				vec->v[v] = 0;
				main_console->Print("modulo vector oob\n");
			}
		}

		// i didnt recognize any problems solved with fmod - we still had lockup
//		if (vec->v[v] > WorldSize)
//			vec->v[v] = (scalar_t)fmod(vec->v[v] - WorldSize, 2 * WorldSize) - WorldSize;

//		if (vec->v[v] < -WorldSize)
//			vec->v[v] = -(scalar_t)fmod(-vec->v[v] + WorldSize, 2 * WorldSize) + WorldSize;
	}
}

scalar_t M_World::DistSq(const vector_t *v1, const vector_t *v2)
{
	vector_t del;

	VectorSub(v1, v2, &del);

	Modulo(&del);

	return VectorLengthSq(&del);
}

void M_World::Delta(const vector_t *dest, const vector_t *src, vector_t *vres)
{
	VectorSub(dest, src, vres);

	Modulo(vres);
}

void M_World::Relative(const vector_t *vec, const vector_t *frame, vector_t *vres)
{
	vector_t reg;
	Delta(vec, frame, &reg);
	VectorAdd(frame, &reg, vres);
}

Texture *M_World::GetSkyTex(void)
{
	if (skymap_use_cubemap) return &skyTex;
	else return &backupSkyTex[0];
}