/*

    File: World.h

    Copyright (C) 2000-2007 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef WORLD_H
#define WORLD_H

#include "../m_math3d/vector.h"
#include "../m_foundation/texture.h"

class M_World {
public:
	M_World();
	~M_World()	{Kill();}

	bool		Init(void);
	void		Kill(void);

	//these should probably be private... but i want them accessible by my entity system (sky cube map reflection texture =) =) =)
	Texture		skyTex;
	Texture		backupSkyTex[6];	//in case cube mapping isnt supported

	Texture		*GetSkyTex(void);	//used in place of directly grabbing the sky tex

	float		fade_start, fade_stop;

	scalar_t	WorldSize;

	void		UpdateFade(void);

	void		Display(void);

	//useful functions
	void		Modulo(vector_t *vec);
	void		Delta(const vector_t *dest, const vector_t *src, vector_t *vres);
	scalar_t	DistSq(const vector_t *v1, const vector_t *v2);
	void		Relative(const vector_t *vec, const vector_t *frame, vector_t *vres);

private:
	void		DrawWorldGrid(void);
	void		DrawSkyCube(void);
};


#endif